BTN_CAPTION_BUILD_QUERY = 'Build Query'
BTN_CAPTION_USE_SAVED_FILTER = 'Use Saved Filter'
BTN_CAPTION_CUSTOM_MGMT = 'Custom Management Command'
DAYS = 'days'
WEEKS = 'weeks'
HOURS = 'hours'
YES = 'Yes'
NO = 'No'
COUNT = 'count'
SUM = 'sum'
SETTINGS_TOKEN = 'TOKEN'
SETTINGS_COMMANDS_SUFFIX = 'COMMANDS_SUFFIX'
SETTINGS_HISTORY_LOOKUP_MODEL_PROPERTY = 'HISTORY_LOOKUP_MODEL_PROPERTY'
SETTINGS_CONVERSATIONS = 'CONVERSATIONS'
SETTINGS_CHAT_ID = 'CHAT_ID'
SETTINGS_MW = 'MIDDLEWARE'
SETTINGS_MW_VIEW = 'view'
SETTINGS_MW_RULES = 'RULES'
SETTINGS_MW_TRIGGER_CODES = 'trigger_codes'
SETTINGS_MW_CONDITIONS = 'conditions'
SETTINGS_MW_CONDITIONS_TYPE = 'type'
SETTINGS_MW_CONDITIONS_FUNC = 'function'
SETTINGS_MW_CONDITIONS_VALUE = 'value'
SETTINGS_MW_CONDITIONS_FIELD = 'field'
SETTINGS_MW_MESSAGE = 'message'
SETTINGS_MW_CONDITIONS_FIELD_VALUE = 'field_value'
LOGGER_NAME = 'django_telegram_bot'
